#include <vector>
#include <queue>
#include <utility>
#include <algorithm>
#include <memory>

#include "Core.h"

namespace rm {

uint32_t get_direction(const Node& from, const Node& to) {
    if (from.col == to.col) return to.row > from.row ? UP : DOWN;
    else return to.col > from.col ? UP : DOWN;
}

Node& Node::operator=(const Node& rhs) {
    row = rhs.row;
    col = rhs.col;
    return *this;
}

bool Node::operator==(const Node& rhs) {
    return row == rhs.row && col == rhs.col;
}

bool Node::operator!=(const Node& rhs) {
    return row != rhs.row || col != rhs.col;
}

bool Node::operator<(const Node& rhs) const {
    return row != rhs.row ? row < rhs.row : col < rhs.col;
}

Grid::Grid(const std::vector<std::vector<bool>>& grid):
    rows(grid.size()),
    cols(grid[0].size()),
    _grid(new grid_t<bool>(grid)) {}

Grid::Grid(std::vector<std::vector<bool>>&& grid):
    rows(grid.size()),
    cols(grid[0].size()),
    _grid(new grid_t<bool>(std::move(grid))) {}

Grid::Grid(const Grid& grid):
    rows(grid._grid->size()),
    cols((*grid._grid)[0].size()),
    _grid(new grid_t<bool>(*grid._grid)) {}

Grid::Grid(Grid&& grid) noexcept:
    rows(grid._grid->size()),
    cols((*grid._grid)[0].size()),
    _grid(grid._grid.release()) {}

std::vector<bool>& Grid::operator[](uint32_t pos) {
    return (*_grid)[pos];
}

bool Grid::is_linked(uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2) const {
    return (*_grid)[x1 + x2][y1 + y2];
}

bool Grid::is_linked(const Node& node1, const Node& node2) const {
    return is_linked(node1.row, node1.col, node2.row, node2.col);
}

Vehicle::Vehicle(const uint32_t speed_row, const uint32_t speed_col, const uint32_t speed_cor):
    speed_row(speed_row),
    speed_col(speed_col),
    speed_cor(speed_cor) {}

Vehicle::Vehicle(const Vehicle& vehicle):
    speed_row(vehicle.speed_row),
    speed_col(vehicle.speed_col),
    speed_cor(vehicle.speed_cor) {}

} // namespace
