#pragma once

#include <vector>
#include <memory>

namespace rm {

template<class T>
using grid_t = std::vector<std::vector<T>>;

struct Node {
    uint32_t row;
    uint32_t col;
    Node& operator=(const Node& rhs);
    bool operator==(const Node& rhs);
    bool operator!=(const Node& rhs);
    // 重载 const 的 operator< 是为了能够用于 std::set
    bool operator<(const Node& rhs) const;
};

enum {
    UP = 0b00,
    DOWN = 0b01,
    LEFT = 0b10,
    RIGHT = 0b11
};

uint32_t get_direction(const Node& from, const Node& to);

// 将原图扩大一倍，把原图 格子的中点 和 边的中点 和 边的交点 作为新图的点
// 此图有两个有趣的性质：
// 1. 原图坐标*2 即新图坐标（坐标需从 0 开始计数）
// 2. 两相邻点坐标相加得到的点在新图中代表这两点相接的边的中点
// 虽然有空间浪费，但是在这个项目里比较无所谓
struct Grid {
    const uint32_t rows;
    const uint32_t cols;
    std::unique_ptr<grid_t<bool>> _grid;
    Grid(const std::vector<std::vector<bool>>& grid);
    Grid(std::vector<std::vector<bool>>&& grid);
    Grid(const Grid& grid);
    Grid(Grid&& grid) noexcept;
    std::vector<bool>& operator[](uint32_t pos);
    // 判断相邻两点是否连通
    bool is_linked(uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2) const;
    // 判断相邻两点是否连通
    bool is_linked(const Node& node1, const Node& node2) const;
};

struct Vehicle {
    // 单位：毫秒/格子
    const uint32_t speed_row;
    const uint32_t speed_col;
    const uint32_t speed_cor; // corner
    Vehicle(const uint32_t speed_row, const uint32_t speed_col, const uint32_t speed_cor);
    Vehicle(const Vehicle& vehicle);
};

} // namespace
