#pragma once

#include <vector>
#include <set>

#include "../Core/Core.h"

namespace rm {

// 在图 grid 上规划一条从点 from 到点 to 的最短路径，返回路径和预估耗时
// current_time 为当前时间相对于比赛起始时间经过的毫秒数
std::pair<std::vector<Node>, uint32_t> plan_one_path(const Grid& grid,
                                                     const std::set<Node>& revolve_area,
                                                     const Vehicle& vehicle,
                                                     const Node& from,
                                                     const Node& to,
                                                     uint32_t current_time,
                                                     bool is_first_map);

// 规划一条从点 from 到点 to 的最短路径，返回路径和预估耗时
std::pair<std::vector<Node>, uint32_t> plan_best_path(const Grid& grid1,
                                                      const Grid& grid2,
                                                      const std::set<Node>& revolve_area1,
                                                      const std::set<Node>& revolve_area2,
                                                      const Vehicle& vehicle,
                                                      const Node& from,
                                                      const Node& to,
                                                      uint32_t current_time);

// 规划到 dsts 中所有点的最佳访问顺序
std::vector<Node> plan_best_order(const Grid& grid1,
                                  const Grid& grid2,
                                  const std::set<Node>& revolve_area1,
                                  const std::set<Node>& revolve_area2,
                                  const Vehicle& vehicle,
                                  const Node& from,
                                  std::vector<Node> dsts,
                                  uint32_t current_time);


} // namespace
