#include <fstream>
#include <vector>
#include <set>
#include <stack>

#include "../Core/Core.h"

namespace rm {
namespace resource {

// 虽然看起来下面这些函数返回时会复制，降低效率，
// 但是 C++ 有强制 NRVO 优化哒！（ C++17 有变 ）

grid_t<bool> get_map(const char* filename) {
    grid_t<bool> grid;
    bool temp;
    std::ifstream fin(filename);
    for (int i = 0; i < 23; ++i) {
        grid.push_back(std::vector<bool>());
        for (int j = 0; j < 23; ++j) {
            fin >> temp;
            grid[i].push_back(temp);
        }
    }
    return grid;
}

std::stack<std::pair<Node, uint32_t>> get_all_dst(const char* filename) {
    std::vector<std::pair<Node, uint32_t>> temp;
    uint32_t row, col, direction;
    std::ifstream fin(filename);
    for (int i = 0; i < 13; ++i) {
        fin >> row >> col >> direction;
        temp.push_back({{row, col}, direction});
    }
    std::stack<std::pair<Node, uint32_t>> all_dst;
    for (auto iter = temp.rbegin(); iter != temp.rend(); ++iter) {
        all_dst.push(*iter);
    }
    return all_dst;
}

std::set<Node> get_revolve_area(const char* filename) {
    std::set<Node> revolve_area;
    uint32_t row, col;
    std::ifstream fin(filename);
    for (int i = 0; i < 13; ++i) {
        fin >> row >> col;
        revolve_area.insert({row, col});
    }
    return revolve_area;
}

Vehicle get_vehicle(const char* filename) {
    uint32_t speed_row, speed_col, speed_cor;
    std::ifstream fin(filename);
    fin >> speed_row >> speed_col >> speed_cor;
    return {speed_row, speed_col, speed_cor};
}

} // namespace
} // namespace
