#include "Serial.h"
#include "Source/serial_device.h"

namespace rm {

SendData& SendData::operator=(const SendData& rhs) {
    next_node = rhs.next_node;
    direction = rhs.direction;
};

ReceiveData& ReceiveData::operator=(const ReceiveData& rhs) {
    curr_node = rhs.curr_node;
    front_unavailable = rhs.front_unavailable;
    found_enemy_cube = found_enemy_cube;
    got_a_cube = got_a_cube;
    go_to_end_node = rhs.go_to_end_node;
};

Serial::Serial(std::string port_name, int baudrate):
    _serial(port_name, baudrate) {
    _serial.Init();
}

void Serial::_read(uint8_t* buf, int len) {
    while (_serial.Read(buf, len) <= 0) usleep(1);
}

void Serial::send(const SendData& send_data) {
    uint8_t data[2] = {0b00000000u
                       | (send_data.next_node.row << 2)
                       | (send_data.next_node.col >> 2),
                       0b01000000u
                       | ((send_data.next_node.col & 0b00000011u) << 4)
                       | (send_data.direction << 2)
                       | send_data.is_at_dst ? 0b00000010u : 0b00000000u};
    _serial.Write(data, 2);
}

ReceiveData Serial::receive() {
    uint8_t data[2];
    do {
        _read(data, 1);
    } while ((data[0] & 0b11000000u) != 0b00000000u);
    _read(&data[1], 1);
    return ReceiveData{{(data[0] & 0b00111100u) >> 2,
                        ((data[0] & 0b00000011u) << 2) | ((data[1] & 0b00110000u) >> 4)},
                       data[1] & 0b00001000u,
                       data[1] & 0b00000100u,
                       data[1] & 0b00000010u,
                       data[1] & 0b00000001u};
}

} // namespace
