#include <vector>
#include <set>
#include <stack>
#include <algorithm>
#include <queue>
#include <thread>
#include <chrono>
#include <iostream>

#include "Core/Core.h"
#include "PathPlanning/PathPlanning.h"
#include "Resource/Resource.h"
#include "Serial/Serial.h"
#include "Time/Time.h"

rm::Grid grid1(rm::resource::get_map("./Resource/map1.txt"));
rm::Grid grid2(rm::resource::get_map("./Resource/map2.txt"));
std::set<rm::Node> revolve_area1(rm::resource::get_revolve_area("./Resource/revolve_area1.txt"));
std::set<rm::Node> revolve_area2(rm::resource::get_revolve_area("./Resource/revolve_area2.txt"));
rm::Vehicle vehicle = rm::resource::get_vehicle("./Resource/vehicle.txt");

rm::Serial serial("/dev/ttyUSB0", 9600);
rm::ReceiveData receive_data = serial.receive();

const bool is_in_left = receive_data.curr_node.col == 0;

std::stack<std::pair<rm::Node, uint32_t>> dst_stack(is_in_left
                                                    ? rm::resource::get_all_dst("./Resource/all_dst_left.txt")
                                                    : rm::resource::get_all_dst("./Resource/all_dst_right.txt"));
rm::Node last_dst = dst_stack.top().first;
bool wait_in_dst = false;

rm::Node start_node = is_in_left ? rm::Node{11, 0} : rm::Node{11, 11};
rm::Node end_node = is_in_left ? rm::Node{0, 9} : rm::Node{0, 2};
uint32_t base_time = rm::get_current_time();

std::vector<std::pair<rm::Node, uint32_t>> known_dsts;
uint32_t known_cube_cnt = 0;
uint32_t loaded_cube_cnt = 0;
std::vector<rm::Node> path = rm::plan_best_path(grid1, grid2, revolve_area1, revolve_area2,
                                                vehicle, start_node, dst_stack.top().first,
                                                rm::get_current_time() - base_time).first;

std::queue<std::pair<rm::Node, uint32_t>> recover_queue;

int main() {
    if (is_in_left) std::cout << "BEGIN IN LEFT\n";
    else std::cout << "BEGIN IN RIGHT\n";
    while (1) {
        while (!recover_queue.empty()) {
            if (rm::get_current_time() - recover_queue.front().second > 5000) {
                rm::Node& node = recover_queue.front().first;
                grid1[node.row][node.col] = 1;
                grid2[node.row][node.col] = 1;
                recover_queue.pop();
            } else break;
        }

        receive_data = serial.receive();
        std::cout << "RECEIVE: " << receive_data.curr_node.row << ", " << receive_data.curr_node.col << "\n";
        std::cout << "DESTINATION: " << dst_stack.top().first.row << ", " << dst_stack.top().first.col << "\n";

        rm::Node& curr_node = receive_data.curr_node;
        rm::Node& last_node = path[0];
        uint32_t curr_direction = get_direction(last_node, curr_node);


        /* -- ͬ -- */

        // ͬȴ׶ΣٴյϢʱ
        // Ϣе curr_node ǰͬӦж wait_in_dst
        if (wait_in_dst) {
            wait_in_dst = false;
        }
        // ߵһͬ
        else if (receive_data.curr_node == dst_stack.top().first) {
            dst_stack.pop();
            last_dst = dst_stack.top().first;
            // صǰͬ򣬸λѵͬ
            serial.send(rm::SendData{curr_node, dst_stack.top().second, true});
            // ͬʱ·滮ͬȴ׶
            wait_in_dst = true;
            continue;
        }


        /* -- ָ -- */

        // ǰ㲻
        if (receive_data.front_unavailable) {
            uint32_t row = curr_node.row * 2;
            uint32_t col = curr_node.col * 2;

            if (curr_direction == rm::UP) --row;
            else if (curr_direction == rm::DOWN) ++row;
            else if (curr_direction == rm::LEFT) --col;
            else if (curr_direction == rm::RIGHT) ++col;

            grid1[row][col] = 0;
            grid2[row][col] = 0;

            // 5 ָ״̬
            recover_queue.push({{row, col}, rm::get_current_time()});
        }
        // ֱȥյ
        else if (receive_data.go_to_end_node) {
            uint32_t temp_direction = is_in_left ? rm::RIGHT : rm::LEFT;
            dst_stack.push({end_node, temp_direction});
        }
        // ֵз  ȡü
        else if (receive_data.found_enemy_cube || receive_data.got_a_cube) {
            if (receive_data.found_enemy_cube) { // ֵз
                uint32_t temp_direction;
                if (dst_stack.top().second == rm::LEFT) temp_direction = rm::RIGHT;
                else if (dst_stack.top().second == rm::RIGHT) temp_direction = rm::LEFT;
                else temp_direction = dst_stack.top().second;
                known_dsts.push_back({{last_dst.row, 11 - last_dst.col}, temp_direction});
                ++known_cube_cnt;
            } else { // ȡü
                ++known_cube_cnt;
                ++loaded_cube_cnt;
            }

            if (known_cube_cnt == 4) {
                std::vector<rm::Node> order;
                for (auto const& pair : known_dsts) {
                    order.push_back(pair.first);
                }
                order = rm::plan_best_order(grid1, grid2, revolve_area1, revolve_area2,
                                            vehicle, curr_node, order,
                                            rm::get_current_time() - base_time);
                std::sort(known_dsts.begin(), known_dsts.end());
                while (1) {
                    bool flag = true;
                    for (std::size_t idx = 0; idx < order.size(); ++idx) {
                        if (known_dsts[idx].first != order[idx]) {
                            flag = false;
                            break;
                        }
                    }
                    if (flag) break;
                    std::next_permutation(known_dsts.begin(), known_dsts.end());
                }

                dst_stack = std::stack<std::pair<rm::Node, uint32_t>>(); // clear
                for (auto iter = known_dsts.rbegin(); iter != known_dsts.rend(); ++iter) {
                    dst_stack.push(*iter);
                }
            }

            if (loaded_cube_cnt == 2) {
                loaded_cube_cnt = 0;
                uint32_t temp_direction = is_in_left ? rm::RIGHT : rm::LEFT;
                dst_stack.push({end_node, temp_direction});
            }
        }


        /* -- 滮·Ϣ -- */

        path = rm::plan_best_path(grid1, grid2, revolve_area1, revolve_area2,
                                  vehicle, curr_node, dst_stack.top().first,
                                  rm::get_current_time() - base_time).first;
        rm::Node next_node = path[1];
        std::cout << "NEXT_NODE: " << next_node.row << ", " << next_node.col << "\n";
        uint32_t next_direction = get_direction(curr_node, next_node);

        // ȴת
        uint32_t moded_time = (rm::get_current_time() - base_time) % 60000;
        uint32_t wait_time = 0;
        if (revolve_area1.find(next_node) != revolve_area1.end()) {
            if (moded_time < 2000) wait_time = 2000 - moded_time;
            else if (moded_time > 28000) wait_time = 62000 - moded_time;
        } else if (revolve_area2.find(next_node) != revolve_area2.end()) {
            if (moded_time < 32000) wait_time = 32000 - moded_time;
            else if (moded_time > 58000) wait_time = 92000 - moded_time;
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(wait_time));

        std::this_thread::sleep_for(std::chrono::milliseconds(1));
        serial.send(rm::SendData{next_node, next_direction, false});
    }
}
